/*
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <bluetooth/services/nus.h>
#include <zephyr/kernel.h>
#include <zephyr/sys/printk.h>
#include "bt_comm.h"
#include "bt_nus.h"

#define BT_NUS_THREAD_PRIORITY   (0u)
#define BT_NUS_THREAD_STACK_SIZE (1024u)

static K_SEM_DEFINE(_gBtNusSem, 0, 1);
static K_THREAD_STACK_DEFINE(_gBtNusThreadStack, BT_NUS_THREAD_STACK_SIZE);
static k_tid_t         _gBtNusThreadId;
static struct k_thread _gBtNusThread;

void _btNusThread(void) {
    uint32_t errCode;

    btNusSetTxCompleted(false);

    printk("BT NUS thread starts running\n");
    const uint32_t mtu = bt_nus_get_mtu(btGetCurrentConn());
    printk("bt_nus_get_mtu(%p)=%u\n", btGetCurrentConn(), mtu);
    btNusSetMtu(mtu);

    while(1) {
        const bt_nus_state_t state = btNusGetState();
        if ((state == BT_NUS_STATE_CAPTURE) ||
            (state == BT_NUS_STATE_READ)) {
           // struct uart_data_t *buf = k_fifo_get(&fifo_uart_rx_data, K_FOREVER);
            errCode = btNusSend();
            if (errCode < 0) {
                printk("Sending NUS data failed\n");
            }

            if ((errCode < 0) || btNusTxBufEmpty()) {
                btNusTxBufClear();
                btNusSetTxCompleted(true);
            }
        } else if (state == BT_NUS_STATE_DISCONNECT) {
            btNusThreadAbort();
        }

        k_usleep(1);
    } 
}

void btNusThreadAbort(void) {
    printk("btNusThreadAbort() e\n");
    k_thread_abort(_gBtNusThreadId);
}

void btNusThreadCreate(void) {
    printk("btNusThreadCreate() e\n");
    _gBtNusThreadId = k_thread_create(&_gBtNusThread,
                                      _gBtNusThreadStack,
                                      BT_NUS_THREAD_STACK_SIZE,
                                      (k_thread_entry_t) _btNusThread,
                                      NULL,
                                      NULL,
                                      NULL,
                                      BT_NUS_THREAD_PRIORITY,
                                      0,
                                      K_NO_WAIT);
}

void btNusThreadResume(void) {
    printk("btNusThreadResume() e\n");
    k_thread_resume(_gBtNusThreadId);
}

void btNusThreadSuspend(void) {
    printk("btNusThreadSuspend() e\n");
    k_thread_suspend(_gBtNusThreadId);
}
