/*
 * Copyright (c) 2016 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/logging/log.h>
#include <zephyr/sys/printk.h>
#include "flash_buf.h"

static bool     _gFlashWriteBufFull = false;
static uint16_t _gFlashWriteHeader  = 0;
static uint16_t _gFlashWriteTail    = 0;
static uint8_t  _gFlashWriteBuf[FLASH_WRITE_BUF_SIZE];

void flashBufInit(void) {
    _gFlashWriteBufFull = false;
    _gFlashWriteHeader  = 0;
    _gFlashWriteTail   = 0;
    memset(_gFlashWriteBuf, 0x00, FLASH_WRITE_BUF_SIZE);
}    

uint16_t flashBufRead(const int len, uint8_t *buf) {
    int i;
    int count = 0;

    for(i=0; i<len; i++) {
        if (_gFlashWriteBufFull || (_gFlashWriteHeader != _gFlashWriteTail))
        {
            buf[i] = _gFlashWriteBuf[_gFlashWriteTail];
            _gFlashWriteTail++;
            if (_gFlashWriteTail >= FLASH_WRITE_BUF_SIZE) {
                _gFlashWriteTail = 0;
            }    
            _gFlashWriteBufFull = false;
            count++;
        } else {
            break;
        }
    }

    return count;
}

uint16_t flashBufWrite(const uint8_t* buf, const int len) {
    int i;
    uint16_t count = 0;
    if (!_gFlashWriteBufFull) {
        for(i=0;i<len;i++) {
            _gFlashWriteBuf[_gFlashWriteHeader] = buf[i];
            _gFlashWriteHeader++;
            if (_gFlashWriteHeader >= FLASH_WRITE_BUF_SIZE) {
                _gFlashWriteHeader = 0;
            }    
            
            count++;
            if (_gFlashWriteHeader == _gFlashWriteTail) 
            {
                printk("norBufferWrite Full \n");
                _gFlashWriteBufFull = true;
                break;
            }
        }
    }

    return count;
}
