/*
 * Copyright (c) 2016 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <zephyr/sys/printk.h>
#include "bt_nus.h"
#include "error.h"
#include "flash_thread.h"
#include "flash_wrap.h"

#define FLASH_THREAD_PRIORITY   (1u)
#define FLASH_THREAD_STACK_SIZE (1024u)

static K_THREAD_STACK_DEFINE(_gFlashThreadStack, FLASH_THREAD_STACK_SIZE);
static bool            _gFlashThreadCreated = false;
static k_tid_t         _gFlashThreadId;
static struct k_thread _gFlashThread;

void _flashThread(void) {
    printk("_flashThread() e\n");
    while(_gFlashThreadCreated) {
        k_msleep(50);
        const bt_nus_state_t nusState = btNusGetState();
        if (nusState == BT_NUS_STATE_MEASURE) {
            if (flashWrapWriteData() == ERROR_NOR_REACH_END) {
                btNusSetState(BT_NUS_STATE_MEASURE_ENDING);
            }
        }
    }
    printk("_flashThread() x\n");
}

void flashThreadAbort(void) {
    printk("flashThreadAbort() e\n");
    if (_gFlashThreadCreated) {
        k_thread_abort(_gFlashThreadId);
        _gFlashThreadCreated = false;
    }
    printk("flashThreadAbort() x\n");
}

void flashThreadCreate(void) {
    printk("flashThreadCreate() e\n");
    if (!_gFlashThreadCreated) {
        _gFlashThreadId = k_thread_create(&_gFlashThread,
                                          _gFlashThreadStack,
                                          FLASH_THREAD_STACK_SIZE,
                                          (k_thread_entry_t) _flashThread,
                                          NULL,
                                          NULL,
                                          NULL,
                                          FLASH_THREAD_PRIORITY,
                                          0,
                                          K_NO_WAIT);
        if (_gFlashThreadId != NULL) {
            _gFlashThreadCreated = true;
        }
    }
    printk("flashThreadCreate() x\n");
}
