/*
 * Copyright (c) 2016 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/logging/log.h>
#include <zephyr/sys/printk.h>
#include "data.h"
#include "error.h"
#include "flash_buf.h"
#include "flash_nor.h"

int flashWrapReadData(const size_t len, uint8_t* buf) {
    const size_t density = flashNorGetDensity();
    off_t offset         = flashNorGetReadOffset();
    
    if (offset >= density-len) {
        printk("End of NOR flash: offset=0x%lx, density=0x%x, len=0x%x\n", offset, density, len);
        return ERROR_NOR_REACH_END;
    }    

    const int ret = flashNorReadData(offset, len, buf);        
    offset += len;
    flashNorSetReadOffset(offset);
    return ret;
}

int flashWrapWriteData(void) {
    int ret = 0;
    static uint8_t buf[FLASH_WRITE_BUF_SIZE];
    const uint16_t len = flashBufRead(2*PPG_ALL_PACKETS_LENGTH, buf);
    
    if (len > 0) {
       // printk("flashBufRead() return %d\n", len);
        ret = flashNorWriteData(len, buf);
        printk("flashNorGetWrittenOffset()=%u\n", flashNorGetWrittenOffset());
    }

    return ret;
}
